//
// Created by hoelshare on 22.05.17.
//

#ifndef GPN_LASERTAG_GAMESERVER_HPP
#define GPN_LASERTAG_GAMESERVER_HPP

#include <GPNBadge.hpp>
#include "BadgeUI.h"
#include "LaserMenuItem.hpp"
#include "PlayerListMenu.hpp"
#include "RemotePlayer.hpp"

#define SERVER_PORT 4803
#define MAX_SRV_CLIENTS 16


class GameServer {
public:
    GameServer(WindowSystem *ui, Badge *badge) : ui(ui), badge(badge) {
        server = new WiFiServer(SERVER_PORT);
        server->begin();
#ifdef DEBUG
        Serial.printf("Server - created\n");
#endif
        serverClients = new WiFiClient[MAX_SRV_CLIENTS]; // initialize the array of clients
#ifdef DEBUG
        Serial.printf("Client Array created\n");
#endif
        sizeMenu = new Menu(2);
#ifdef DEBUG
        Serial.printf("Menu Created\n");
#endif
        sizeMenu->addMenuItem(new MenuItem("TEAM", [=]() {
            this->setTeamPlay(1);
            ui->closeCurrent();
            // TODO open Menu anzTeam

        }));
        sizeMenu->addMenuItem(new MenuItem("SINGLE", [=]() {
#ifdef DEBUG
            Serial.println("Single selected");
#endif
            this->setTeamPlay(0);
            ui->closeCurrent();
#ifdef DEBUG
            Serial.println("UI Closed");
#endif
        }));
#ifdef DEBUG
        Serial.printf("Open menu\n");
#endif
//        ui->open(sizeMenu);
//        ui->dispatchInput(badge->getJoystickState());
//        ui->draw();
#ifdef DEBUG
        Serial.printf("Opened menu\n");
#endif
        playerList = new PlayerListMenu(9);
        LaserMenuItem* item = new LaserMenuItem([]() {}, nullptr);
        item->setText("Playerlist");
        playerList->addMenuItem(item);
        playerList->setLeftAction([=]() { this->kick(); });
        playerList->setRightAction([=]() { this->secureQuestion(); });
        playerList->setEnterAction([=]() { this->sendIP();  });
        ui->open(playerList);
        ui->dispatchInput(badge->getJoystickState());
        ui->draw();
#ifdef DEBUG
        Serial.printf("Actions added\n");
#endif
//        playerList->addMenuItem(new MenuItem(player->getNickname(), []() {}));
//        ui->open(playerList);

#ifdef DEBUG
        Serial.printf("Playerlist opend\n");
#endif
    }

    virtual void update();

protected:
    void setTeamPlay(char isTeam);

    void startGame();

    virtual void secureQuestion();

    virtual void handleWelcome(WiFiClient *client, short iPlayer);

    virtual void kick();

    void sendAll(String value);

private:
    WindowSystem *ui;
    Badge *badge;
    Menu *sizeMenu;
    PlayerListMenu *playerList;
    PlayerListMenu *secureMenu;
    WiFiServer *server;  // the server object
    WiFiClient *serverClients; // the clients connected to the server
    short numConnections = 0;
    bool gameStarted = false;
    void sendIP();
    RemotePlayer rPlayers[MAX_SRV_CLIENTS];
    String buildPlayerList();
};

#endif //GPN_LASERTAG_GAMESERVER_HPP
